/////////////////////////////////////////////////////////////////////////////////
//						       International Karate +						   //
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
// Auteur : Alexandre Bocquillon   											   //
// Librairie graphique : OpenGl et SDL										   //
// Language utilis : C++											           //
// Note : Ceci est mon premier programme avec la lib SDL, OpenGL et le C++.	   //
// 		  Pour compiler correctement ce programme, il faut les librairies	   //
// 		  "SDL 1.2", "SDL_Image","SDL_ttf" et fmod.							   //
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
// Pour ceux qui connaissent dja ce jeux dvelopp  l'origine sur Amstrad    //
// puis sur Amiga, seul le niveau bonus a t programm.					   //
// Il n'y a donc pas de combat entre diffrents combattants(pour l'instant...).//
// Pour les autres qui ne connaissent pas le jeux, le but est de 		       //
// repousser les projectiles le plus longtemps possible afin d'ammasser le	   //
// maximum de points.														   //
// Pour orienter le personnage, il faut utiliser les flches directionnelles   //
/////////////////////////////////////////////////////////////////////////////////


#include "main.h"
#include "Variables.h"


int main (int argc, char *argv[])
{

	//Initialisation OpenGl et SDL
    if(SDL_Init(SDL_INIT_VIDEO)!=0) {
		cerr << "Probleme pour initialiser SDL :" << SDL_GetError() << endl;
        exit(2);
    }
    SDL_WM_SetCaption(GAME_TITLE, NULL);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);
    SDL_SetVideoMode(RESOLUTION_X, RESOLUTION_Y, 0, SDL_OPENGL);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, RESOLUTION_X, RESOLUTION_Y, 0);
    glEnable (GL_TEXTURE_2D);
	glClearColor(0.0,0.0,0.0,0.0);

	//Initialisation de FMOD
	if(!FSOUND_Init(44100, 16, 0 )) {
        cerr << "Probleme pour initialiser FMOD" << endl;
        exit(2);
    }

	//Initialisation de SDL_ttf pour le texte
	if(TTF_Init()==-1) {
        cerr << "Probleme pour initialiser SDL_ttf" << endl;
        exit(2);
    }

	//On passe par la SDL lorsque l'on quittera le programme
	atexit(SDL_Quit);

	//Initialisation du timer pour gnrer des nombres alatoires
	srand((unsigned) time(NULL));

	//Taux d'affichage
	SDL_initFramerate(&manager);
    SDL_setFramerate(&manager,FPS);

	//On affiche le menu
	Menu();

	//Fermeture de FMOD
    FSOUND_Close();

	return 0;
}


void Menu()
{

	//Variables pour grer les vnements
	SDL_Event event;
	bool done=false;
	Uint8* keys;

	//On raffiche le titre 
	SDL_WM_SetCaption(GAME_TITLE, NULL);

	//Chargement de l'animation
	menu.loadbackground("Medias/Backgrounds/ik.png");
	menu.loadsound("Medias/Sounds/Intro.mp3");

	//On efface l'cran
	glClear(GL_COLOR_BUFFER_BIT);

	if(Sound){menu.playsound();}

	//Animation d'introduction
	for (int posX = 0 ; posX < RESOLUTION_X + 10 ; posX = posX + 10 * (100/FPS) )
	{

		menu.drawbackground(posX);
		SDL_framerateDelay(&manager);
		SDL_GL_SwapBuffers();

	}
	
	SDL_setFramerate(&manager,10);

	//Attente d'un vnement clavier
	while(!done)
	{

		while(SDL_PollEvent(&event))

		switch(event.type)
		{

		case SDL_QUIT:
			exit(0);
			break;

		case SDL_KEYDOWN :
			keys = SDL_GetKeyState(NULL);
			if ( keys[SDLK_ESCAPE] ) {exit(0);}
			if ( keys[SDLK_RETURN] ) {done=1;}
			break;

		}

		SDL_framerateDelay(&manager);
		SDL_GL_SwapBuffers();

	}

	SDL_setFramerate(&manager,FPS);

	//Initialisation de la partie
	NewGame();


}


void NewGame()
{
	
	//Remise  zro du compteur des scores
	level.resetscore();

	//On charge les images et les sons en mmoire
	LoadImages();
	LoadSounds();

	//Paramtres d'initialisation de l'instance joueur
	player.setpos(260,300);
	player.setaction(RIGHT, MIDDLE);

	//Cration d'une boule
	CreateNewBall(0);

	//On commence la partie (boucle principale)
	Run();

}


void LoadImages()
{

	level.loadbackground("Medias/Backgrounds/Background.png");
	player.load("Medias/Sprites/Karateka.png",360,320,3,2);

}

void LoadSounds()
{

	level.loadmusic("Medias/Music/IKPlus.mp3",true);
	player.loadsound("Medias/Sounds/Die.mp3");

}



void Run()
{

	//variables pour grer les vnements
	SDL_Event event;
	bool done=false;
	Uint8* keys;

	//String avec le nombre de FPS
	char caption[30];

	//Lecture de la musique
	if(Music){level.playmusic(true);}


	//variables de gestion du texte
	Ctext text;
	text.LoadFont("Medias/Polices/times.ttf", 48, 255, 0, 0);
	text.SetPosition(350, 150);

	//Mmorisation du temps  laquelle la partie a commenc
	InitialTime = SDL_GetTicks();

	//Boucle principale
	while(!done)
	{

		//Gestion des vnements
		while(SDL_PollEvent(&event))
		{

			switch(event.type)
			{

			case SDL_QUIT:
				exit(0);
				break;

			case SDL_KEYDOWN :
				keys = SDL_GetKeyState(NULL);
				if ( keys[SDLK_ESCAPE] ) {done=1;}
				if ( keys[SDLK_UP] ) { player.setaction(player.way,UP); }
				if ( keys[SDLK_DOWN] ) { player.setaction(player.way,DOWN); }
				if ( keys[SDLK_LEFT] && player.way==LEFT ) { player.setaction(LEFT,MIDDLE); }
				if ( keys[SDLK_LEFT] && player.way==RIGHT ) { player.setaction(LEFT,player.movement); }
				if ( keys[SDLK_RIGHT] && player.way==RIGHT) { player.setaction(RIGHT,MIDDLE); }
				if ( keys[SDLK_RIGHT] && player.way==LEFT) { player.setaction(RIGHT,player.movement); }

				break;

			}

		}

		//On met  jour les boules et on test si collision
		for (IBall = LBall.begin (); IBall != LBall.end(); ++IBall)
		{
			(*IBall)->update();
			if(TestCollision()){done=1;break;}
		}

		//Mise  jour du nbre de boules et du dcor
		UpdateBallLife();

		//On dessine le backscreen
		Draw();

		//Affichage du nombre de FPS
		sprintf(caption, "IK+ ; FPS : %d ; Score : %i", calculFPS(), level.returnscore());
		SDL_WM_SetCaption(caption, NULL);

		//text.DrawText("hello");
		SDL_framerateDelay(&manager);

		//On permute les cran (double buffuring)
		SDL_GL_SwapBuffers();

	}

	//Arrt de la musique
	if(Music){level.playmusic(false);}

	//Effacement des boules
	for (IBall = LBall.begin (); IBall != LBall.end();)
	{
		delete *IBall;
		IBall = LBall.erase(IBall);
	}

	//Remise a zro des variables grant l'augmentation de vitesse des boules
	NumberOf10s = 0;
	SpeedIncrement=0;

	//On revient au menu
	Menu();


}


void Draw()
{

	level.drawbackground();

	player.draw();

	for (IBall = LBall.begin (); IBall != LBall.end(); ++IBall)
	(*IBall)->draw();

}





int calculFPS()
{

	static int curtime=0;
	static int checktime=0;
	static double curfps;
	static double fpscounter=0;

	curtime=SDL_GetTicks();

	if (curtime>checktime)
	{
		checktime=curtime+1000;
		curfps=fpscounter;
		fpscounter = 0;
	} else
	{
		fpscounter+=1;
	}

	return (int)curfps;

}



int TestCollision()
{

	static const int offcet = 30;

	//test si boule percute le joueur
	if ( (*IBall)->returnposX() < ( player.pos_X + player.returnsizeX() - offcet ) && ( (*IBall)->returnposX() + (*IBall)->returnsizeX() ) > (player.pos_X + offcet) ){PlayerDie();return 1;}

	//test si contact entre le bouclier et la boule
	if ( (*IBall)->returnposX() < ( player.pos_X + player.returnsizeX() ) && ( (*IBall)->returnposX() + (*IBall)->returnsizeX() ) > player.pos_X )
	{

		//test si reception
		if ( player.way == RIGHT && (*IBall)->returnWay()== -1 || player.way == LEFT && (*IBall)->returnWay()== 1)
		{

			switch(player.movement)
			{
			case UP:
				if ( (*IBall)->returnposY() < (player.pos_Y + 55) ){(*IBall)->changeway();level.updatescore();}
				break;
			case MIDDLE:
				if ( (*IBall)->returnposY() > (player.pos_Y + 20) && (*IBall)->returnposY() < (player.pos_Y + 90) ){(*IBall)->changeway();level.updatescore();}
				break;
			case DOWN:
				if ( (*IBall)->returnposY() > (player.pos_Y + 60) ){(*IBall)->changeway();level.updatescore();}
				break;

			}

		}

	}

	return 0;

}


void PlayerDie()
{

	int done=0;
	int InitialTime = SDL_GetTicks();

	//Cri du joueur
	if(Sound){player.playsound();}

	//On charge et affiche le personnage mort
	player.load("Medias/Sprites/Die.png",156,32,1,1);
	player.setpos(255,435);

	//On attend quelques secondes pour laisser sortir les boules de l'cran
	while( (SDL_GetTicks() - InitialTime) < 3000 )
	{

		for (IBall = LBall.begin (); IBall != LBall.end(); ++IBall)
		{
			(*IBall)->update();
		}

		Draw();

		SDL_framerateDelay(&manager);

		SDL_GL_SwapBuffers();

	}

}


void UpdateBallLife()
{

	//Augmente le nombre de boules
	if ( (SDL_GetTicks() - InitialTime) / 10000 > NumberOf10s && NumberOf10s < 2)
	{ ++NumberOf10s; CreateNewBall(0); }

	if ( (SDL_GetTicks() - InitialTime) / 10000 > NumberOf10s && NumberOf10s >= 2)
	{ ++NumberOf10s; SpeedIncrement = SpeedIncrement + 0.5f; }

	//Si la boule quitte l'cran, on la dtruit et on en cr une autre
	for (IBall = LBall.begin (); IBall != LBall.end();)
	{
		if ( (*IBall)->returnposX()  < -60 || (*IBall)->returnposX() > RESOLUTION_X )
		{ delete *IBall; IBall = LBall.erase(IBall);CreateNewBall(SpeedIncrement);}else{++IBall;}
	}

}


void CreateNewBall(float SpeedIncrement)
{

	int way=0;
	int posX=0;

	//Cration d'une nouvelle boule
	LBall.push_back(new CBall());

	//Gnration alatoire du cot d'arrive de la boule
	if(rand()%2 == 1){way = 1;posX=-60;}else{way =-1;posX=RESOLUTION_X;}
	LBall.back()->initparam(posX,430,way,SpeedIncrement);


	//Gnration alatoire de la couleur de la boule
	switch (1+rand()%5)
	{
		case 1:
			LBall.back()->load("Medias/Sprites/Ball1.png",44,44,1,1);
			break;
		case 2:
			LBall.back()->load("Medias/Sprites/Ball2.png",44,44,1,1);
			break;
		case 3:
			LBall.back()->load("Medias/Sprites/Ball3.png",44,44,1,1);
			break;
		case 4:
			LBall.back()->load("Medias/Sprites/Ball4.png",44,44,1,1);
			break;
		case 5:
			LBall.back()->load("Medias/Sprites/Ball5.png",44,44,1,1);
			break;
	}

	LBall.back()->loadsound("Medias/Sounds/Hit.mp3");
}
